program StegoBMP;

{$APPTYPE CONSOLE}

uses
  Graphics,
  Classes,
  SysUtils,
  DateUtils,
  ZLib,
  Steganografia in 'Steganografia.pas';

var
  InBMP: TBitmap;
  Data: TFileStream;
  Stego: TStegoStream;
  zStream: TStream;
  A, B, C, D: string;
  Bits, Size: Integer;
  Speed: Double;
  Compr: Single;
  Inizio, Fine: TDateTime;
  
begin

  WriteLn('StegoBMP Version 4.5.1');
  WriteLn('(C)opyRight AndSoft Inc., 2001');
  WriteLn;
  WriteLn('ZLib version ', zlib_Version);
  WriteLn;

  if ParamCount in [1..4] then begin

    A := ParamStr(1);
    B := ParamStr(2);
    C := ParamStr(3);
    D := ParamStr(4);

    try
      InBMP := TBitmap.Create;
      InBMP.LoadFromFile(A);
      Bits := StrToIntDef(D, 1);
      Stego := TStegoStream.Create(InBMP, Bits);
      WriteLn('Max encode data size:   ', Stego.MaxSize: 9, ' (', (Stego.MaxSize / 1024.0): 7: 2, ' kB)');
      WriteLn('Bits per color:         ', Stego.BitsXColor: 9);

      case ParamCount of
        2: begin
          Data := TFileStream.Create(B, fmCreate);
          zStream := TDecompressionStream.Create(Stego);
          if zStream.Read(Size, SizeOf(Size)) <> SizeOf(Size) then
            Size := 0;
          WriteLn('ZLib file size is:      ', Stego.Size: 9, ' (', (Stego.Size / 1024.0): 7: 2, ' kB)');
          WriteLn('Data file size is:      ', Size: 9, ' (', (Size / 1024.0): 7: 2, ' kB)');

          WriteLn;
          Write('Please wait while decoding...');
          Inizio := Time;
          Data.CopyFrom(zStream, Size);
          Fine := Time;
          zStream.Free;
          try
            Speed := Data.Size / MilliSecondsBetween(Fine, Inizio);
          except
            Speed := 0;
          end;
          WriteLn('Done.');
          WriteLn;
          WriteLn(Data.Size, ' bytes in ', FormatDateTime('ss:zzz', Fine - Inizio), ' sec (', Speed: 7: 2, ' kB/s).');
        end;
        4: begin
          Stego.SetSize(0);
          zStream := TCompressionStream.Create(clMax, Stego);
          Data := TFileStream.Create(C, fmOpenRead);
          Size := Data.Size;
          WriteLn('Requested data size is: ', Size: 9, ' (', (Size / 1024.0): 7: 2, ' kB)');

          WriteLn;
          Write('Please wait while encoding...');
          Inizio := Time;
          zStream.Write(Size, SizeOf(Size));
          zStream.CopyFrom(Data, Size);
          Fine := Time;
          Compr := (zStream as TCompressionStream).CompressionRate;
          zStream.Free;
          try
            Speed := Data.Size / MilliSecondsBetween(Fine, Inizio);
          except
            Speed := 0;
          end;
          WriteLn('Done.');
          WriteLn;
          WriteLn(Stego.Size, ' bytes in ', FormatDateTime('ss:zzz', Fine - Inizio), ' sec (', Speed: 7: 2, ' kB/s).');
          WriteLn('Compression rate: ', Compr: 5: 2, '%.');
          Stego.WriteStego(B);
        end;
      else
        zStream := TDecompressionStream.Create(Stego);
        if zStream.Read(Size, SizeOf(Size)) <> SizeOf(Size) then
          Size := 0;
        zStream.Free;
        WriteLn('ZLib file size is:      ', Stego.Size: 9, ' (', (Stego.Size / 1024.0): 7: 2, ' kB)');
        WriteLn('Data file size is:      ', Size: 9, ' (', (Size / 1024.0): 7: 2, ' kB)');
      end;

    except
      ShowException(ExceptObject, ExceptAddr);
    end;
    Stego.Free;
    Data.Free;
  end else begin
    WriteLn('Usage: (encode) stegobmp InBMP.bmp OutBMP.bmp datafile BitsXColor');
    WriteLn('       (decode) stegobmp InBMP.bmp datafile');
    WriteLn('       (info)   stegobmp InBMP.bmp');
    WriteLn;
    WriteLn('All parameters are mandatory.');
  end;
end.
